/****************************************************************************************
 * Copyright (c) 2010 Maximilian Kossick <maximilian.kossick@googlemail.com>            *
 *                                                                                      *
 * This program is free software; you can redistribute it and/or modify it under        *
 * the terms of the GNU General Public License as published by the Free Software        *
 * Foundation; either version 2 of the License, or (at your option) any later           *
 * version.                                                                             *
 *                                                                                      *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
 *                                                                                      *
 * You should have received a copy of the GNU General Public License along with         *
 * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
 ****************************************************************************************/

#ifndef AMAROK_DEFAULTAPPLICATIONCONTROLLER_H
#define AMAROK_DEFAULTAPPLICATIONCONTROLLER_H

#include "statemanagement/ApplicationController.h"

namespace Amarok
{
    class DefaultApplicationController : public ApplicationController
    {
        Q_OBJECT
    public:
        explicit DefaultApplicationController( QObject *parent );
        ~DefaultApplicationController() override;

    public Q_SLOTS:
        void start() override;
        void shutdown() override;

    private:
        void initSqlStorage();
        void initEngineController();
        void initCollectionManager();
        void initCollectionLocationDelegate();
    };
}

#endif
