/***************************************************************************
 *   Copyright (c) 2010 Casey Link <unnamedrambler@gmail.com>              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef TESTTRACKORGANIZER_H
#define TESTTRACKORGANIZER_H

#include "CollectionTestImpl.h"

#include <QObject>

class TrackOrganizer;

class TestTrackOrganizer : public QObject
{
Q_OBJECT
public:
    TestTrackOrganizer();
    
private Q_SLOTS:
  void init();
  void cleanup();
  void testBasic();

private:
    Meta::TrackList makeTracks( int numTracks );
    Meta::TrackPtr makeMockTrack( const QString &trackName, const QString &artistName, const QString &albumName, int tracknumber );
  
    TrackOrganizer* mTrackOrganizer;  
    Collections::CollectionTestImpl *mColl;
     Meta::TrackList mTracks;
};

#endif // TESTTRACKORGANIZER_H
