/****************************************************************************************
 * Copyright (c) 2010 Maximilian Kossick <maximilian.kossick@googlemail.com>            *
 *                                                                                      *
 * This program is free software; you can redistribute it and/or modify it under        *
 * the terms of the GNU General Public License as published by the Free Software        *
 * Foundation; either version 2 of the License, or (at your option) any later           *
 * version.                                                                             *
 *                                                                                      *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
 * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
 *                                                                                      *
 * You should have received a copy of the GNU General Public License along with         *
 * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
 ****************************************************************************************/

#ifndef ARTISTHELPER_H
#define ARTISTHELPER_H

#include "amarok_export.h"

#include <QString>

namespace ArtistHelper
{
    /**
     * Return the best guess of the album artist.
     *
     * Guessing algorithm includes extracting artist from composer for classical tracks,
     * falling back to trackArtist if albumArtist is empty etc. Returns empty string for
     * tracks that are believed to belong into a compilation.
     */
    AMAROK_EXPORT QString bestGuessAlbumArtist( const QString &albumArtist, const QString &trackArtist,
                                  const QString &genre, const QString &composer );

    /**
      * This helper function will determine the actual track artist.
      * It takes into account "A featuring B" strings, in which case
      * the actual track artist would be A.
      */
    AMAROK_EXPORT QString realTrackArtist( const QString &trackArtistTag );
}

#endif
